"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var epoxy_1 = require("../../epoxy");
var isGlob = require("is-glob");
var isValidPath = require("is-valid-path");
var graphql_1 = require("graphql");
var glob = require("glob");
var fs_1 = require("fs");
var extract_document_string_from_code_file_1 = require("../../utils/extract-document-string-from-code-file");
var GQL_EXTENSIONS = ['.graphql', '.graphqls', '.gql'];
var INVALID_SCHEMA_KINDS = [graphql_1.Kind.OPERATION_DEFINITION, graphql_1.Kind.FRAGMENT_DEFINITION];
function isGraphQLFile(globPath) {
    return GQL_EXTENSIONS.some(function (ext) { return globPath.endsWith(ext); });
}
function loadSchemaFile(filepath) {
    var content = fs_1.readFileSync(filepath, 'utf-8');
    if (content && content.trim() !== '') {
        if (/^\#.*import /i.test(content.trimLeft())) {
            var importSchema = require('graphql-import').importSchema;
            return importSchema(filepath);
        }
        var foundDoc = extract_document_string_from_code_file_1.extractDocumentStringFromCodeFile(new graphql_1.Source(content, filepath));
        if (foundDoc) {
            return foundDoc;
        }
        return content;
    }
    else {
        console['warn']("Empty schema file found: \"" + filepath + "\", skipping...");
    }
    return null;
}
var SchemaFromTypedefs = /** @class */ (function () {
    function SchemaFromTypedefs() {
    }
    SchemaFromTypedefs.prototype.canHandle = function (globPath) {
        return isGlob(globPath) || (isValidPath(globPath) && isGraphQLFile(globPath));
    };
    SchemaFromTypedefs.prototype.handle = function (globPath) {
        var globFiles = glob.sync(globPath, { cwd: process.cwd() });
        if (!globFiles || globFiles.length === 0) {
            throw new Error("Unable to find matching files for glob: " + globPath + " in directory: " + process.cwd());
        }
        var filesContent = globFiles.map(function (filePath) { return ({ filePath: filePath, content: loadSchemaFile(filePath) }); }).filter(function (file) {
            if (!file.content) {
                return false;
            }
            var node = graphql_1.parse(file.content);
            var invalidSchemaDefinitions = node.definitions.filter(function (def) { return INVALID_SCHEMA_KINDS.includes(def.kind); });
            if (invalidSchemaDefinitions.length === 0) {
                return true;
            }
            else {
                console['warn']("File \"" + file.filePath + "\" was filtered because it contains an invalid GraphQL schema definition!");
                return false;
            }
        });
        if (filesContent.length === 0) {
            throw new Error("All found files for glob expression \"" + globPath + "\" are not valid or empty, please check it and try again!");
        }
        return epoxy_1.mergeGraphQLSchemas(filesContent.map(function (f) { return f.content; }));
    };
    return SchemaFromTypedefs;
}());
exports.SchemaFromTypedefs = SchemaFromTypedefs;
//# sourceMappingURL=schema-from-typedefs.js.map