"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
function isObjectTypeDefinition(obj) {
    return obj && obj.kind === 'ObjectTypeDefinition';
}
function parseDirectiveValue(value) {
    switch (value.kind) {
        case graphql_1.Kind.INT:
            return parseInt(value.value);
        case graphql_1.Kind.FLOAT:
            return parseFloat(value.value);
        case graphql_1.Kind.BOOLEAN:
            return Boolean(value.value);
        case graphql_1.Kind.STRING:
        case graphql_1.Kind.ENUM:
            return value.value;
        case graphql_1.Kind.LIST:
            return value.values.map(function (v) { return parseDirectiveValue(v); });
        case graphql_1.Kind.OBJECT:
            return value.fields.reduce(function (prev, v) {
                var _a;
                return (__assign({}, prev, (_a = {}, _a[v.name.value] = parseDirectiveValue(v.value), _a)));
            }, {});
        case graphql_1.Kind.NULL:
            return null;
        default:
            return null;
    }
}
function getFieldsWithDirectives(documentNode) {
    var result = {};
    var allTypes = documentNode.definitions.filter(isObjectTypeDefinition);
    for (var _i = 0, allTypes_1 = allTypes; _i < allTypes_1.length; _i++) {
        var type = allTypes_1[_i];
        var typeName = type.name.value;
        for (var _a = 0, _b = type.fields; _a < _b.length; _a++) {
            var field = _b[_a];
            if (field.directives && field.directives.length > 0) {
                var fieldName = field.name.value;
                var key = typeName + "." + fieldName;
                var directives = field.directives.map(function (d) { return ({
                    name: d.name.value,
                    args: (d.arguments || []).reduce(function (prev, arg) {
                        var _a;
                        return (__assign({}, prev, (_a = {}, _a[arg.name.value] = parseDirectiveValue(arg.value), _a)));
                    }, {}),
                }); });
                result[key] = directives;
            }
        }
    }
    return result;
}
exports.getFieldsWithDirectives = getFieldsWithDirectives;
//# sourceMappingURL=get-fields-with-directives.js.map